#include "RTC.h"
#include <EEPROM.h>

#if (UPDATE_INTERVAL > 255)
#error "UPDATE_INTERVAL must be either a multiple of 60, or less than 256 seconds"
#endif

RTC_DATA_ATTR bool WatchFeatures::RTC::m_timerSet = false;
RTC_DATA_ATTR bool WatchFeatures::RTC::m_initialTimer = true;

void WatchFeatures::RTC::Get(tmElements_t & tm)
{
  m_rtcPcf.getDateTime();
  tm.Year = y2kYearToTm(m_rtcPcf.getYear());
  tm.Month = m_rtcPcf.getMonth();
  tm.Day = m_rtcPcf.getDay();
  tm.Wday = m_rtcPcf.getWeekday() + 1; // TimeLib has Wday range of 1-7, but PCF8563 stores day of week in 0-6 range
  tm.Hour = m_rtcPcf.getHour();
  tm.Minute = m_rtcPcf.getMinute();
  tm.Second = m_rtcPcf.getSecond();
}

void WatchFeatures::RTC::Set(tmElements_t tm)
{
  time_t t = makeTime(tm); // make and break to calculate tm.Wday
  breakTime(t, tm);
  // day, weekday, month, century(1=1900, 0=2000), year(0-99)
  m_rtcPcf.setDateTime(tm.Day, tm.Wday - 1, tm.Month, 0, tmYearToY2k(tm.Year), tm.Hour, tm.Minute, tm.Second);
}

uint64_t WatchFeatures::RTC::GetTimestamp()
{
  return m_rtcPcf.getTimestamp64();
}

void WatchFeatures::RTC::SetTimer()
{
  if (!m_timerSet) {
    Resync();
  }
}

void WatchFeatures::RTC::OffsetTime(tmElements_t & tm, int offsetInSeconds)
{
  int year = tm.Year;
  int month = tm.Month;
  int day = tm.Day;
  int hour = tm.Hour;
  int minute = tm.Minute;
  int second = tm.Second;

  // adjust for offset
  second += offsetInSeconds;
  if (second >= 60 || second < 0) {
    minute += second / 60;
    second = second % 60;
  }

  if (minute >= 60 || minute < 0) {
    hour += minute / 60;
    minute = minute % 60;
  }

  if (hour >= 24 || hour < 0) {
    day += hour / 24;
    hour = hour % 24;
  }

  bool leapYear = year % 4 == 0;
  if (leapYear) {
    if (month == 2 && day > 29) {
      month++;
      day = day % 29;
    }
  } else {
    if (month == 2 && day > 28) {
      month++;
      day = day % 28;
    }
  }

  if (month == 4 || month == 6 || month == 9 || month == 11) {
    if (day > 30) {
      month++;
      day = day % 30;
    }
  } else {
    if (day > 31) {
      month++;
      day = day % 31;
    }
  }

  if (month > 12 || month < 0) {
    year++;
    month = month % 12;
  }

  tm.Year = year;
  tm.Month = month;
  tm.Day = day;
  tm.Hour = hour;
  tm.Minute = minute;
  tm.Second = second;
  time_t t = makeTime(tm); // make and break to calculate tm.Wday
  breakTime(t, tm);
}

unsigned int WatchFeatures::RTC::GetDayOfYear(tmElements_t & tm)
{
  unsigned int dayOfYear = 1;
  for (int i = 1; i < tm.Month; i++) {
    switch (i) {
      case 2:
        dayOfYear += (tm.Year % 4 == 0) ? 29 : 28;
        break;
      case 4:
      case 6:
      case 9:
      case 11:
        dayOfYear += 30;
        break;
      default:
        dayOfYear += 31;
        break;
    }
  }

  dayOfYear += tm.Day;
  return dayOfYear;
}

unsigned int WatchFeatures::RTC::DaysDifference(tmElements_t & tm1, tmElements_t & tm2)
{
  unsigned int dayOfYear1 = GetDayOfYear(tm1);
  unsigned int dayOfYear2 = GetDayOfYear(tm2);
  unsigned int days = 0;

  if (tm1.Year == tm2.Year) {
    days = dayOfYear2 - dayOfYear1;
  } else {
    days = 365 - dayOfYear1 + dayOfYear2;
    for (int i = tm1.Year + 1; i < tm2.Year; i++) {
      days += (i % 4 == 0) ? 366 : 365;
    }
  }

  return days;
}


// TODO: implement more advanced wakeup logic, i.e. > 255 seconds that are not a multiple of 60
bool WatchFeatures::RTC::CheckWakeup()
{
  if(m_initialTimer) {
    m_initialTimer = false;

    byte frequency, interval;
    if (UPDATE_INTERVAL % 60 == 0) {
      frequency = TMR_1MIN; // 1 minute
      interval = UPDATE_INTERVAL / 60;
    } else {
      frequency = TMR_1Hz; // 1 second
      interval = UPDATE_INTERVAL;
    }

    // Timer doesn't work reliably unless it's cleared first
    m_rtcPcf.clearTimer();
    m_rtcPcf.setTimer(interval, frequency, true);

    return true;
  }

  return true;
}

std::string WatchFeatures::RTC::GetDateString()
{
  return std::string(m_rtcPcf.formatDate(RTCC_DATE_ISO8601));
}

void WatchFeatures::RTC::Resync()
{
  m_rtcPcf.getDateTime();
  // Sleep just long enough to get to a multiple of the update interval, makes updates happen on exact turn of the minute
  int seconds = UPDATE_INTERVAL - (m_rtcPcf.getSecond() % UPDATE_INTERVAL);

  if (seconds < 0) {
    seconds = 0;
  }

  // Timer doesn't work reliably unless it's cleared first
  m_rtcPcf.clearTimer();

  if (seconds == 0) {
    // If there's no time to wait, just call CheckWakeup() immediately and it'll set the repeating timer
    m_timerSet = true;
    m_initialTimer = true;
    CheckWakeup();
  } else {
    m_rtcPcf.setTimer(seconds, TMR_1Hz, false);

    m_timerSet = true;
    m_initialTimer = true;
  }

  seconds = 0;
}